uniform sampler2D Texture0, Texture1, Texture2, Texture3;
varying vec3 Normal;
varying float fogFactor;
varying vec4 Position;

vec4 CelShading(int l)
{
	// Get texture colors
	vec4 color = gl_LightModel.ambient * texture2D(Texture0, vec2(gl_TexCoord[0]));
	vec4 texel1 = texture2D(Texture1, vec2(gl_TexCoord[0]));
	vec4 texel2 = texture2D(Texture2, vec2(gl_TexCoord[0]));
	vec4 texel3 = texture2D(Texture3, vec2(gl_TexCoord[0]));

	float t1Total = texel1.r + texel1.g + texel1.b;
	float t2Total = texel2.r + texel2.g + texel2.b;
	float t3Total = texel3.r + texel3.g + texel3.b;

	if (t2Total > 0.0) color = gl_LightModel.ambient * vec4(0.5, 0.0, 0.0, 1.0) * t2Total / 3.0;
	else if (t1Total >= 1.0) color = gl_LightModel.ambient * vec4(0.5, 0.0, 0.0, 1.0);
	else if (t3Total >= 1.0) color = gl_LightModel.ambient * vec4(0.4, 0.3725, 0.3215, 1.0);

	// Light orientation
	vec3 LightDir = vec3(gl_LightSource[l].position-Position);
	float dotProduct = max(dot(normalize(LightDir), normalize(Normal)), 0.0);

	// Light attenuation
	float dist = length(LightDir);
	float att = 1.0 / (gl_LightSource[l].constantAttenuation + gl_LightSource[l].quadraticAttenuation * dist * dist);
	att *= 500;

	// 'Tooning'
	float toonFactor = dotProduct;
	toonFactor *= att;
	if (toonFactor < 0.2) toonFactor = clamp(toonFactor, 0.0, 0.05);
	else if (toonFactor < 0.5) toonFactor = clamp(toonFactor, 0.2, 0.35);
	else if (toonFactor < 0.95) toonFactor = clamp(toonFactor, 0.5, 0.65);
	else toonFactor = clamp(toonFactor, 0.95, 1.0);

	// Final color
	vec4 diffuse = gl_LightSource[l].diffuse;
	diffuse /= vec4(128, 128, 128, 1.0);

	if (t2Total > 0.0) color += toonFactor * diffuse * vec4(0.5, 0.0, 0.0, 1.0) * t2Total / 3.0;
	else if (t1Total >= 1.0) color += toonFactor * diffuse * vec4(0.5, 0.0, 0.0, 1.0);
	else if (t3Total >= 1.0) color += toonFactor * diffuse * vec4(0.4, 0.3725, 0.3215, 1.0);
	else color += toonFactor * diffuse * texture2D(Texture0, vec2(gl_TexCoord[0]));

	return color;
}


void main()
{
	vec4 color = vec4(0.0, 0.0, 0.0, 0.0);

	for (int i = 0; i < gl_MaxLights; i++)
	{
		color += CelShading(i);
	}

	// Mask flash
	vec4 texColor = texture2D(Texture0, vec2(gl_TexCoord[0]));
	if (texColor.r >= 1.0 && texColor.g <= 0.0 && texColor.b <= 0.0)
	{
		color = vec4(0.25, 0.0, 0.0, 1.0);
	}

	// Rim light
	vec3 eyePos = normalize(vec3(-Position));
	float fresnel = max(dot(normalize(Normal), eyePos), 0.0);
	if (fresnel < 0.4)
	{
		float rim = clamp((0.4 - fresnel) / 0.4, 0.0, 1.0);
		color += color * rim;
	}

	color = mix(gl_Fog.color, color, fogFactor);
	gl_FragColor = color;
}
